extends Node2D


enum BOSS_HORIZONTAL_PATTERN {
	APPEAR,
	GO_TO_LEFT,
	GO_TO_RIGHT,
	GO_TO_CENTER,
}
enum BOSS_VERTICAL_PATTERN {
	GO_TO_TOP,
	GO_TO_DOWN,
	STAY,
}
enum BOSS_SHOOT_PATERN {
	WAVE,
	SPLASH,
	STAR,
}

const ENEMY_BULLET = preload("res://Scenes/EnemyBullet.tscn")
const EXPLOSION = preload("res://Scenes/ShipExplosion.tscn")
const LIMITS = [25, 810, 230, 150]

var life_points = 100
var inverted = false
var cannons_openned = false
var flash_cooldown_time = 0.1
var flash_cooldown = 0.1
var flashing = false
var horizontal_pattern = BOSS_HORIZONTAL_PATTERN.APPEAR
var vertical_pattern = BOSS_VERTICAL_PATTERN.STAY
var move_cooldown = 0
var horizontal_speed = 100
var vertical_speed = 60
var new_horizontal_speed = 100
var new_vertical_speed = 60

var shoot_pattern = BOSS_SHOOT_PATERN.WAVE
var shoot_cooldown = 0
var shoot_cooldown_time = 0.5
var shoots_remain = 0
var shoot_step = 0
var shoot_speed = 200

var score_hit = 20
var score_death = 2500

var exploading = false
var exploading_time = 3
var explosion_cooldown = 0
var explosion_cooldown_time = 0.25
var explosion_side = 1

var xxx = 3
var yyy = 1


# Ready ========================================================================
func _ready():
	pass


# Process ======================================================================
func _process(delta):

	if flashing:
		flash_cooldown -= delta
		if flash_cooldown <= 0:
			flash(false)


	# Movement -----------------------------------------------------------------

	if not exploading:

		horizontal_speed = lerp(horizontal_speed, new_horizontal_speed, 0.1)
		vertical_speed = lerp(vertical_speed, new_vertical_speed, 0.1)

		move_cooldown -= delta
		if horizontal_pattern != BOSS_HORIZONTAL_PATTERN.APPEAR and move_cooldown <= 0:
			move()

		if horizontal_pattern == BOSS_HORIZONTAL_PATTERN.APPEAR:
			if global_position.y < 120:
				translate(Vector2(0, 1) * 100 * delta)
			else:
				move()
		else:
			if global_position.y <= LIMITS[0] or global_position.x >= LIMITS[1] or global_position.y >= LIMITS[2] or global_position.x <= LIMITS[3]:
				move()
		
		if horizontal_pattern == BOSS_HORIZONTAL_PATTERN.GO_TO_CENTER:
			if global_position.x >= 479 and global_position.x <= 481:
				move()
			else:
				global_position.x += horizontal_speed * delta * (1 if global_position.x <= 480 else -1)

		if horizontal_pattern == BOSS_HORIZONTAL_PATTERN.GO_TO_LEFT:
			if global_position.x > LIMITS[3]:
				global_position.x -= horizontal_speed * delta

		if horizontal_pattern == BOSS_HORIZONTAL_PATTERN.GO_TO_RIGHT:
			if global_position.x < LIMITS[1]:
				global_position.x += horizontal_speed * delta

		if vertical_pattern == BOSS_VERTICAL_PATTERN.GO_TO_TOP:
			if global_position.y > LIMITS[0]:
				global_position.y -= vertical_speed * delta

		if vertical_pattern == BOSS_VERTICAL_PATTERN.GO_TO_DOWN:
			if global_position.y < LIMITS[2]:
				global_position.y += vertical_speed * delta

	
		# Shooting -----------------------------------------------------------------

		shoot_cooldown -= delta
		if horizontal_pattern != BOSS_HORIZONTAL_PATTERN.APPEAR and cannons_openned and shoot_cooldown <= 0 and shoots_remain > 0:
			shoot_cooldown = shoot_cooldown_time
			shoots_remain -= 1
			if shoot_pattern == BOSS_SHOOT_PATERN.WAVE:
				var x = -1
				while x <= 1:
					x += 0.1
					shoot($CannonLeft.global_position, Vector2(x, 1), shoot_speed)
					shoot($CannonRight.global_position, Vector2(x*-1, 1), shoot_speed)
			if shoot_pattern == BOSS_SHOOT_PATERN.SPLASH:
				var x = -1
				while x <= 1:
					x += 0.1
					shoot($CannonLeft.global_position, Vector2(x, 1), shoot_speed)
					shoot($CannonLeft.global_position, Vector2(x, -1), shoot_speed)
					shoot($CannonLeft.global_position, Vector2(1, x), shoot_speed)
					shoot($CannonLeft.global_position, Vector2(-1, x), shoot_speed)
					shoot($CannonRight.global_position, Vector2(x*-1, 1), shoot_speed)
					shoot($CannonRight.global_position, Vector2(x*-1, -1), shoot_speed)
					shoot($CannonRight.global_position, Vector2(1, x*-1), shoot_speed)
					shoot($CannonRight.global_position, Vector2(-1, -x*-1), shoot_speed)
			if shoot_pattern == BOSS_SHOOT_PATERN.STAR:
				shoot_step = -1 if shoot_step >= 1 else (shoot_step + 0.25)
				shoot($CannonLeft.global_position, Vector2(shoot_step, 1), shoot_speed)
				shoot($CannonRight.global_position, Vector2(shoot_step*-1, 1), shoot_speed)

		xxx -= delta
		if xxx <= 0:
			xxx = 3
			if cannons_openned:
				close_cannons()
			else:
				open_cannons()

	# Super Explosion ----------------------------------------------------------
	else:
		exploading_time -= delta
		if exploading_time > 0:
			translate(Vector2(0, 1) * 30 * delta)
			explosion_cooldown -= delta
			if explosion_cooldown <= 0:
				explosion_cooldown = explosion_cooldown_time
				var explosion = EXPLOSION.instance()
				get_parent().add_child(explosion)
				explosion.global_position.x = global_position.x + ((randi()%192) * 1 if (randi()%2 == 0) else -1)
				explosion.global_position.y = global_position.y + ((randi()%103) * 1 if (randi()%2 == 0) else -1)
				explosion.explode(inverted)
				if modulate.a >= 0.1:
					modulate.a -= 0.1
				flash(true)
		else:
			explode()




# Actions ======================================================================

# Universe changed -------------------------------------------------------------
func universe_changed(is_inverted):
	invert(is_inverted)


# Invert -----------------------------------------------------------------------
func invert(is_inverted):
	inverted = is_inverted
	$Bodywork/Sprite.frame = 0 if inverted else 1
	$Shield/Sprite.frame = 0 if inverted else 1


# Move -------------------------------------------------------------------------
func move():
	randomize()
	move_cooldown = rand_range(2, 4)
	new_horizontal_speed = rand_range(80, 120)
	new_vertical_speed = rand_range(60, 100)
	var current_horizontal_pattern = horizontal_pattern
	while horizontal_pattern == current_horizontal_pattern:
		randomize()
		match randi()%3:
			0:
				horizontal_pattern = BOSS_HORIZONTAL_PATTERN.GO_TO_CENTER
			1:
				horizontal_pattern = BOSS_HORIZONTAL_PATTERN.GO_TO_LEFT
			2:
				horizontal_pattern = BOSS_HORIZONTAL_PATTERN.GO_TO_RIGHT
	randomize()
	match randi()%3:
		0:
			vertical_pattern = BOSS_VERTICAL_PATTERN.STAY
		1:
			vertical_pattern = BOSS_VERTICAL_PATTERN.GO_TO_TOP
		2:
			vertical_pattern = BOSS_VERTICAL_PATTERN.GO_TO_DOWN


# Set shoot pattern ------------------------------------------------------------
func set_shoot_pattern():
	
	var new_shoot_pattern = shoot_pattern
	while new_shoot_pattern == shoot_pattern:
		randomize()
		match randi()% 3:
			0:
				shoot_pattern = BOSS_SHOOT_PATERN.WAVE
				shoot_cooldown = 0
				shoot_cooldown_time = 0.3
				shoot_step = 0
				shoots_remain = 3
				shoot_speed = 200
			1:
				shoot_pattern = BOSS_SHOOT_PATERN.SPLASH
				shoot_cooldown = 0
				shoot_cooldown_time = 0.25
				shoot_step = 0
				shoots_remain = 2
				shoot_speed = 150
			2:
				shoot_pattern = BOSS_SHOOT_PATERN.STAR
				shoot_cooldown = 0
				shoot_cooldown_time = 0.3
				shoot_step = 0
				shoots_remain = 20
				shoot_speed = 300


# Shoot ------------------------------------------------------------------------
func shoot(position, direction, speed):
	var bullet = ENEMY_BULLET.instance()
	get_parent().add_child(bullet)
	bullet.set_aim(direction, speed, inverted)
	bullet.global_position = position


# Explode ----------------------------------------------------------------------
func explode():
	get_tree().call_group('Camera', 'screen_shake', 3)
	get_parent().get_parent().set_act()
	queue_free()


# Take damage ------------------------------------------------------------------
func take_damage():
	life_points -= 1
	if life_points > 0:
		get_parent().get_parent().update_score(false, score_hit)
		flash(true)
	else:
		get_parent().get_parent().update_score(false, score_death)
		var player_ship = get_parent().get_parent().get_node("PlayerShip")
		player_ship.invencible = true
		exploading = true
		# explode()


# Flash -----------------------------------------------------------------------
func flash(active):
	flashing = active
	flash_cooldown = flash_cooldown_time
	$Bodywork/Sprite.material.set_shader_param('active', active)
	$Bodywork/Sprite.material.set_shader_param('inverted', not inverted)
	$Shield/Sprite.material.set_shader_param('active', active)
	$Shield/Sprite.material.set_shader_param('inverted', not inverted)


# Open cannons -----------------------------------------------------------------
func open_cannons():
	if not cannons_openned:
		set_shoot_pattern()
		$Shield/Animation.play("Open")
		yield(get_tree().create_timer(0.5), 'timeout')
		cannons_openned = true


# Open cannons -----------------------------------------------------------------
func close_cannons():
	if cannons_openned:
		cannons_openned = false
		$Shield/Animation.play("Close")
		

# Signals ======================================================================

# When cannons are hitted ------------------------------------------------------
func _on_AreaGuns_area_entered(area:Area2D):
	if cannons_openned:
		take_damage()
