extends Node2D


const BULLET = preload("res://Scenes/EnemyBullet.tscn")
const EXPLOSION = preload("res://Scenes/ShipExplosion.tscn")
const LIMITS = [-555, 1420, 810, -480]

export var life_points = 3

var alive = true
var inverted = false
var pattern
var type
var can_shoot = true
var shoot_cooldown_time = 0.25
var shoot_cooldown = 0
var shoots_remain = 3
var configure_shoot = true
var cannon_x = 0
var cannon_y = 0
var cannon_x_sum = true
var cannon_y_sum = true
var flash_cooldown_time = 0.1
var flash_cooldown = 0.1
var flashing = false
var score_hit = 10
var score_death = 100

signal destroyed(intensity)


# Ready ========================================================================
func _ready():
	pass
	# set_type(EnemyType.LIZARD)


# Process ======================================================================
func _process(delta):

	if flashing:
		flash_cooldown -= delta
		if flash_cooldown <= 0:
			flash(false)
	
	if pattern == EnemyPattern.FREE_FALL:
		translate(Vector2(0, 1) * 100 * delta)
		if global_position.y >= 153:
			shoot_cooldown -= delta
			if configure_shoot:
				shoots_remain = 4
				shoot_cooldown_time = 0.35
				configure_shoot = false
			if shoot_cooldown <= 0 and shoots_remain > 0:
				shoot_cooldown = shoot_cooldown_time
				shoots_remain -= 1
				shoot(Vector2(1, -1).normalized(), 200)
				shoot(Vector2(1, 1).normalized(), 200)
				shoot(Vector2(-1, 1).normalized(), 200)
				shoot(Vector2(-1, -1).normalized(), 200)
	
	if pattern == EnemyPattern.HURRICANE:
		translate(Vector2(0, 1) * 100 * delta)
		if global_position.y >= 25:
			shoot_cooldown -= delta
			if configure_shoot:
				shoots_remain = 50
				shoot_cooldown_time = 0.05
				cannon_x = 0
				cannon_y = -1
				configure_shoot = false
			if shoot_cooldown <= 0 and shoots_remain > 0:
				shoot_cooldown = shoot_cooldown_time
				cannon_x += (delta * 10) * (1 if cannon_x_sum else -1)
				cannon_y += (delta * 10) * (1 if cannon_y_sum else -1)
				
				if cannon_x > 1:
					cannon_x_sum = false
				if cannon_x < -1:
					cannon_x_sum = true				
				if cannon_y > 1:
					cannon_y_sum = false
				if cannon_y < -1:
					cannon_y_sum = true
				
				shoots_remain -= 1
				shoot(Vector2(cannon_x, cannon_y), 200)

	if pattern == EnemyPattern.TWISTER:
		translate(Vector2(0, 1) * 120 * delta)
		if global_position.y >= 50:
			shoot_cooldown -= delta
			if configure_shoot:
				shoots_remain = 10
				shoot_cooldown_time = 0.25
				configure_shoot = false
			if shoot_cooldown <= 0 and shoots_remain > 0:
				shoot_cooldown = shoot_cooldown_time
				shoots_remain -= 1
				shoot(Vector2(1, 0), 100)
				shoot(Vector2(-1, 0), 100)
				shoot(Vector2(1, 1), 100)
				shoot(Vector2(-1, 1), 100)
				shoot(Vector2(0, -1), 100)
				
	if global_position.x > LIMITS[1] or global_position.x < LIMITS[3] or global_position.y > LIMITS[2] or global_position.y < LIMITS[0]:
		explode(false)


# Actions ======================================================================

# Set the enemy type -----------------------------------------------------------
func set_type(enemy_type):
	type = enemy_type
	$Chassis/EnemyLizard.visible = true if type == EnemyType.LIZARD else false
	$Chassis/EnemyCrocodile.visible = true if type == EnemyType.CROCODILE else false
	$Chassis/EnemyKomodo.visible = true if type == EnemyType.KOMODO else false
	match enemy_type:
		EnemyType.LIZARD:
			life_points = 3
			score_hit = 10
			score_death = 100
		EnemyType.CROCODILE:
			life_points = 5
			score_hit = 15
			score_death = 150
		EnemyType.KOMODO:
			life_points = 10
			score_hit = 20
			score_death = 200


# Set the enemy pattern --------------------------------------------------------
func set_pattern(enemy_pattern):
	pattern = enemy_pattern
	configure_shoot = true


# Invert -----------------------------------------------------------------------
func invert(is_inverted):
	inverted = is_inverted
	$Chassis/EnemyLizard.frame = 1 if inverted else 0
	$Chassis/EnemyCrocodile.frame = 1 if inverted else 0
	$Chassis/EnemyKomodo.frame = 1 if inverted else 0


# Universe changed -------------------------------------------------------------
func universe_changed(is_inverted):
	invert(is_inverted)


# Take damage ------------------------------------------------------------------
func take_damage():
	life_points -= 1
	if life_points > 0:
		flash(true)
		get_parent().get_parent().update_score(false, score_hit)
	else:
		get_parent().get_parent().update_score(false, score_death)
		explode()


# Explode ----------------------------------------------------------------------
func explode(shew=true):
	flash(false)
	if shew:
		var explosion = EXPLOSION.instance()
		get_parent().add_child(explosion)
		explosion.global_position = global_position
		explosion.explode(inverted)
		get_tree().call_group('Camera', 'screen_shake', 3)
	queue_free()


# Shoot ------------------------------------------------------------------------
func shoot(direction, speed):
	var bullet = BULLET.instance()
	get_parent().add_child(bullet)
	bullet.set_aim(direction, speed, inverted)
	bullet.global_position = global_position


# Flash ------------------------------------------------------------------------
func flash(active):
	flashing = active
	flash_cooldown = flash_cooldown_time
	match type:
		EnemyType.LIZARD:
			$Chassis/EnemyLizard.material.set_shader_param('inverted', inverted)
			$Chassis/EnemyLizard.material.set_shader_param('active', active)
		EnemyType.CROCODILE:
			$Chassis/EnemyCrocodile.material.set_shader_param('inverted', inverted)
			$Chassis/EnemyCrocodile.material.set_shader_param('active', active)
		EnemyType.KOMODO:
			$Chassis/EnemyKomodo.material.set_shader_param('inverted', inverted)
			$Chassis/EnemyKomodo.material.set_shader_param('active', active)


# Signals ======================================================================

# When the game change the universe --------------------------------------------
func _on_Game_invert_universe(is_inverted):
	invert(is_inverted)


# When a player's bullet hit the enemy ship ------------------------------------
func _on_Area_area_entered(area):
	take_damage()
