extends Node2D

enum ResetState {
	READY,
	GO,
}

enum GameAct {
	MAIN_MENU,
	INTRODUCTION,
	TUTORIAL,
	IT_IS_EASY,
	IT_IS_SHOWTIME,
	IT_IS_TWISTER,
	BOSS_FIGHT,
	YOU_WIN,
}

const SCREENPLAY = [
	GameAct.MAIN_MENU,
	GameAct.INTRODUCTION,
	GameAct.TUTORIAL,
	GameAct.IT_IS_EASY,
	GameAct.IT_IS_EASY,
	GameAct.IT_IS_EASY,
	GameAct.IT_IS_SHOWTIME,
	GameAct.IT_IS_SHOWTIME,
	GameAct.IT_IS_SHOWTIME,
	GameAct.IT_IS_TWISTER,
	GameAct.IT_IS_TWISTER,
	GameAct.IT_IS_TWISTER,
	GameAct.BOSS_FIGHT,
	GameAct.YOU_WIN,
]

export(NodePath) var GameCamera
export(NodePath) var PlayerShip
export(NodePath) var Speaker
export(NodePath) var Interface
export(NodePath) var MainMenu

var inverted = false
var game_state = GameState.WAITING
var act_count = 0
var current_act = 0
var screenplay = SCREENPLAY[0]
var reset_state = ResetState.READY
var reset_max_speed = 300
var reset_speed = 300
var next_act_time = 10
var ship_interface_on_screen = false
var main_menu_index = 0
var tutorial_played = false
var tutorial_state = TutorialState.MOVE

var say_ready = true
var say_go = true

var score = 0
var high_score = 0

signal invert_universe(is_inverted)


# Ready ========================================================================
func _ready():
	GameCamera = get_node(GameCamera)
	PlayerShip = get_node(PlayerShip)
	Speaker = get_node(Speaker)
	Interface = get_node(Interface)
	MainMenu = get_node(MainMenu)
	PlayerShip.invert(inverted)
	game_state = GameState.WAITING
	act_count = len(SCREENPLAY)
	screenplay = SCREENPLAY[0]
	current_act = -1
	set_act()


# Precess ======================================================================
func _process(delta):
	
	if Input.is_action_just_pressed("ui_full_screen"):
		OS.window_fullscreen = not OS.window_fullscreen
	
	if screenplay == GameAct.MAIN_MENU:
		pass
	
	if screenplay == GameAct.TUTORIAL:
		if tutorial_played:
			set_act()
		else:

			# Learn to invert
			if tutorial_state == TutorialState.INVERT:
				if Input.is_action_just_pressed("ui_invert"):
					tutorial_played = true
					$Tutorial/Animation.play("Leave")
					set_act()

			# Learn to dash
			if tutorial_state == TutorialState.DASH:
				if Input.is_action_just_pressed("ui_dash"):
					tutorial_state = TutorialState.INVERT
					$Tutorial.change(tutorial_state)

			# Learn to shoot
			if tutorial_state == TutorialState.SHOOT:
				if Input.is_action_just_pressed("ui_shoot"):
					tutorial_state = TutorialState.DASH
					$Tutorial.change(tutorial_state)
			
			# Learn to move
			if tutorial_state == TutorialState.MOVE:
				var input_vector = Vector2.ZERO
				input_vector.x = Input.get_action_strength("ui_right") - Input.get_action_strength("ui_left")
				input_vector.y = Input.get_action_strength("ui_down") - Input.get_action_strength("ui_up")
				if input_vector != Vector2.ZERO:
					tutorial_state = TutorialState.SHOOT
					$Tutorial.change(tutorial_state)

	
	if game_state == GameState.RESET:
		if PlayerShip.global_position.y >= 400:
			reset_speed = lerp(reset_speed, 100, 0.1)
			PlayerShip.global_position.y -= (reset_speed * delta)
			if say_ready:
				say_ready = false
				Speaker.play_voice("ready")
		else:
			PlayerShip.go()
			game_state = GameState.GAME
			if say_go:
				say_go = false
				Speaker.play_voice("go")
			set_act()

	
	if game_state == GameState.GAME and PlayerShip.state != PlayerState.DEAD:
		next_act_time -= delta
		if next_act_time <= 0:
			set_act()


# Actions ======================================================================

# Invert the Universe ----------------------------------------------------------
func invert():
	inverted = not inverted
	PlayerShip.invert(inverted)
	$Enemies.invert(inverted)
	if inverted:
		$Background/BackgroundBlack.visible = false
		$Background/BackgroundWhite.visible = true
	else:
		$Background/BackgroundBlack.visible = true
		$Background/BackgroundWhite.visible = false
	for enemy in $Enemies.get_children():
		if enemy.has_method('universe_changed'):
			enemy.universe_changed(inverted)
	emit_signal('invert_universe', inverted)


# Clear battlefield ------------------------------------------------------------
func clear_battlefield(shew=false):
	for enemy in $Enemies.get_children():
		if shew:
			if enemy.has_method("explode"):
				enemy.explode()
			else:
				enemy.queue_free()
		else:
			enemy.queue_free()


# Set act ----------------------------------------------------------------------
func set_act():
	current_act += 1
	if current_act < act_count:
		screenplay = SCREENPLAY[current_act]

		if screenplay == GameAct.MAIN_MENU:
			print("No Menu Principal")
			game_state = GameState.MAIN_MENU
			MainMenu.enter()
			if ship_interface_on_screen:
				ship_interface_on_screen = false
				Interface.get_node("InterfaceBars/Animation").play("Leave")
				Interface.get_node("Score/Animation").play("Leave")
			if inverted:
				invert()
		
		if screenplay == GameAct.INTRODUCTION:
			print("Na Introdução")
			inverted = false
			say_ready = true
			say_go = true
			game_state = GameState.RESET
			reset_speed = reset_max_speed
			PlayerShip.reset()
			Speaker.play_music("level")
			if not ship_interface_on_screen:
				ship_interface_on_screen = true
				Interface.get_node("InterfaceBars/Animation").play("Enter")
				Interface.get_node("Score/Animation").play("Enter")
			emit_signal('invert_universe', inverted)

		if screenplay == GameAct.TUTORIAL:
			if not tutorial_played:
				$Tutorial/Animation.play("Enter")
			next_act_time = 3600
		
		if screenplay == GameAct.IT_IS_EASY:
			$Enemies.stage_it_is_easy()
			next_act_time = 7.5
		
		if screenplay == GameAct.IT_IS_SHOWTIME:
			$Enemies.stage_it_is_showtime()
			next_act_time = 8
		
		if screenplay == GameAct.IT_IS_TWISTER:
			$Enemies.stage_it_is_twister()
			next_act_time = 8

		if screenplay == GameAct.BOSS_FIGHT:
			$Enemies.stage_boss_fight()
			next_act_time = 3600
		
		if screenplay == GameAct.YOU_WIN:
			$PlayerShip.win()
			next_act_time = 3600


# Update score -----------------------------------------------------------------
func update_score(reset_score=false, points=0):
	if reset_score:
		score = 0
	else:
		score += points
		if score > high_score:
			high_score = score
	$ShipInterface/Score/ScorePoints.text = str(score)
	$ShipInterface/Score/HighScorePoints.text = str(high_score)


# Update final score -----------------------------------------------------------
func update_final_score():
	$ShipInterface/WinData/Score.text = ("Score: %s" % str(score))


# Retry ------------------------------------------------------------------------
func retry():
	if screenplay == GameAct.YOU_WIN:
		$ShipInterface/WinData/Animation.play("Leave")
	current_act = 0
	clear_battlefield(true)
	update_score(true, 0)
	yield(get_tree().create_timer(1), 'timeout')
	set_act()


# Give up ----------------------------------------------------------------------
func give_up():
	if screenplay == GameAct.YOU_WIN:
		$ShipInterface/WinData/Animation.play("Leave")
	current_act = -1
	clear_battlefield(true)
	update_score(true, 0)
	yield(get_tree().create_timer(1), 'timeout')
	set_act()
